@@ -18,6 +18,7 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "../config.h"   // for HAVE_LIBLAPACK
 #include <iostream>
 
 #include "APL_types.hh"
@@ -28,7 +29,7 @@
 
 using namespace std;
 
-#define USE_LAPACK
+#if HAVE_LIBLAPACK   // liblapack is installed.found
 
 /**
 
@@ -52,10 +53,12 @@ Copyright (c) 2006-2011 The University of Colorado Denver.  All rights
 
 **/
 
-#ifdef USE_LAPACK
-
 /// An integer.
+#ifdef __LP64__
+typedef int integer;
+#else
 typedef long integer;
+#endif
 
 /// A Floating point number.
 typedef double doublereal;
@@ -102,7 +105,7 @@ integer       info;
              &lwork, rwork, &info);
 
      if (info)
-        CERR << "info = " << (int)info << " in pass 1 of dgelsy_()" << endl;
+        CERR << "info = " << info << " in pass 1 of dgelsy_()" << endl;
 
      lwork = 10 + integer(work[0].r);
    }
@@ -114,7 +117,7 @@ integer       info;
              &lwork, rwork, &info);
 
      if (info)
-        CERR << "info = " << (int)info << " in pass 2 of dgelsy_()" << endl;
+        CERR << "info = " << info << " in pass 2 of dgelsy_()" << endl;
    }
 
    return info;
@@ -141,7 +144,7 @@ integer       info;
              &lwork, &info);
 
      if (info)
-        CERR << "info = " << (int)info << " in pass 1 of dgelsy_()" << endl;
+        CERR << "info = " << info << " in pass 1 of dgelsy_()" << endl;
      lwork = 10 + integer(work[0]);
    }
 
@@ -152,7 +155,7 @@ integer       info;
              &lwork, &info);
 
      if (info)
-        CERR << "info = " << (int)info << " in pass 2 of dgelsy_()" << endl;
+        CERR << "info = " << info << " in pass 2 of dgelsy_()" << endl;
    }
 
    return info;
@@ -231,7 +234,7 @@ Value_P Z = new Value(shape_Z, LOC);
 //-----------------------------------------------------------------------------
 //-----------------------------------------------------------------------------
 
-#else
+#if 0   // divide_matrix() without liblapack
 
 /*
    this is an attempt to remove the dependency on liblapack in order to
@@ -400,5 +403,24 @@ Tmatrix<T>::divide(Tvector<T> & ZZ, const Tvector<T> & AA) const
 }
 //-----------------------------------------------------------------------------
 
-#endif
+#endif // divide_matrix() without liblapack
+
+#else    // not HAVE_LIBLAPACK
+
+#warning liblapack not found or not installed. ⌹ will not work.
+
+Value_P
+divide_matrix(ShapeItem rows, ShapeItem cols_A, Value_P A, ShapeItem cols_B,
+              Value_P B, const Shape & shape_Z, APL_Float qct)
+{
+   CERR <<
+"function divide_matrix() aka. ⌹  returns DOMAIN ERROR because\n"
+"library liblapack was not found when GNU APL was compiled. To fix this\n" 
+"install liblapack from http://www.netlib.org/lapack/ and re-run\n"
+"./configure, make, and make install." << endl;
+
+   DOMAIN_ERROR;
+}
+
+#endif   // HAVE_LIBLAPACK
 
