@@ -69,7 +69,7 @@ Nabla::edit()
    if (const char * loc = start())
       {
         CERR << "bad editor command '" << first_command
-             << "' : problem at " << loc << endl;
+             << "' : problem '" << loc << "'" << endl;
        NABLA_ERROR;
       }
 
@@ -78,9 +78,8 @@ Nabla::edit()
    Log(LOG_nabla)   CERR << "Nabla(" << fun_name << ")..." << endl;
    while (!do_close)
        {
-         char prompt[40];
-         current_line.print_prompt(prompt, sizeof(prompt));
-         const char * line = (const char *)Input::get_user_line_1(prompt);
+         const UCS_string prompt = current_line.print_prompt();
+         const char * line = (const char *)Input::get_user_line_1(&prompt);
          const UTF8_string utf(line);
          const UCS_string ucs(utf);
          if (const char * loc = parse_oper(ucs, false))
@@ -209,13 +208,13 @@ Unicode cc = c.next();
    // [n]                                           (goto)
    // text                                          (override text)
 
-   if (cc == UNI_NABLA)       // ∇
+   if (cc == UNI_NABLA)       // initial ∇
       {
         do_close = true;
         return 0;
       }
 
-   if (cc == UNI_DEL_TILDE)   // ⍫
+   if (cc == UNI_DEL_TILDE)   // initial ⍫
       {
         locked = true;
         do_close = true;
@@ -230,7 +229,15 @@ Unicode cc = c.next();
         return 0;
       }
 
-skip:
+   // a loop over multiple commands, like
+   // [2⎕4] [∆5]
+   //
+   // only the last command is executed; the previous commands are discarded/
+   //
+command_loop:
+
+   // at this point, [ was seen and skipped
+
    ecmd = ECMD_NOP;
    edit_from.clear();
    edit_to.clear();
@@ -240,8 +247,12 @@ skip:
    //
    if (Avec::is_digit(c.get()))   edit_from = parse_lineno(c);
 
-   // operation
+   // operation, which is one of:
    //
+   // [⎕   show
+   // []   edit
+   // [∆   delete
+   // [→   abandon
    switch (c.get())
       {
         case UNI_QUAD_QUAD:     ecmd = ECMD_SHOW;     c.next();   break;
@@ -268,7 +279,7 @@ again:
 
    if (c.get() == UNI_ASCII_MINUS)   // range
       {
-        if (got_minus)   return "error: second -  at "LOC;
+        if (got_minus)   return "error: second -  at " LOC;
         got_minus = true;
         edit_from = edit_to;
         c.next();   // eat the -
@@ -277,32 +288,41 @@ again:
 
    if (c.next() != UNI_ASCII_R_BRACK)   return LOC;
 
+   // at this point we have parsed an editor command, like:
+   //
+   // [from ⎕ to]
+   // [from ∆ to]
+   // [from]
+
    while (c.get() == UNI_ASCII_SPACE)   c.next();
 
    if (c.get() == UNI_ASCII_L_BRACK)   // another command: ignore previous
       {
          c.next();   // eat the [
-         goto skip;
+         goto command_loop;
       }
 
-   while ((cc = c.next()) != Invalid_Unicode)
-      {
-        if (cc == UNI_NABLA)
-           {
-             do_close = true;
-             break;
-           }
+   // copy the rest to current_text and set do_close if the first character
+   // is ∇ or ⍫
+   //
+   while ((cc = c.next()) != Invalid_Unicode)   current_text += cc;
 
-        if (cc == UNI_DEL_TILDE)
+   if (current_text.size())
+      {
+        if (current_text[0] == UNI_NABLA)
            {
+             current_text.clear();
+             do_close = true;
+           }
+        else if (current_text[0] == UNI_DEL_TILDE)
+           {
+             current_text.clear();
              locked = true;
              do_close = true;
-             break;
            }
-
-        current_text += cc;
       }
 
+
    return 0;
 }
 //-----------------------------------------------------------------------------
@@ -576,20 +596,21 @@ LineLabel::print(ostream & out) const
    out << "]";
 }
 //-----------------------------------------------------------------------------
-void
-LineLabel::print_prompt(char * buf, size_t size) const
+UCS_string
+LineLabel::print_prompt() const
 {
-int idx = snprintf(buf, size - 4, "[%d", ln_major);
+UCS_string ret("[");
+   ret.append_number(ln_major);
 
    if (ln_minor.size())
       {
-        buf[idx++] = '.';
-        loop(s, ln_minor.size())   buf[idx++] = char(ln_minor[s]);
+        ret += Unicode('.');
+        loop(s, ln_minor.size())   ret += Unicode(char(ln_minor[s]));
       }
 
-   buf[idx++] = ']';
-   buf[idx++] = ' ';
-   buf[idx++] = 0;
+   ret += Unicode(']');
+   ret += Unicode(' ');
+   return ret;
 }
 //-----------------------------------------------------------------------------
 void
