@@ -384,7 +384,6 @@ Token::print_quad(ostream & out) const
 UCS_string
 Token::canonical(PrintStyle style) const
 {
-
 UCS_string ucs;
 
    switch(get_Class())
@@ -442,6 +441,21 @@ UCS_string ucs;
    return ucs;
 }
 //-----------------------------------------------------------------------------
+UCS_string
+Token::tag_name() const
+{
+   switch(get_tag())
+      {
+#define TD(tag, _tc, _tv, _id) case tag: return UCS_string( #tag );
+#include "Token.def"
+      }
+
+char cc[40];
+   snprintf(cc, sizeof(cc), "0x%X", get_tag());
+UCS_string ucs(cc);
+   return ucs;
+}
+//-----------------------------------------------------------------------------
 int
 Token::error_info(UCS_string & ucs) const
 {
