@@ -32,6 +32,7 @@
 #include "Command.hh"
 #include "Common.hh"
 #include "Input.hh"
+#include "LibPaths.hh"
 #include "Logging.hh"
 #include "main.hh"
 #include "Output.hh"
@@ -55,13 +56,14 @@ bool do_not_echo = false;
 //-----------------------------------------------------------------------------
 /// initialize subsystems
 void
-init()
+init(const char * argv0)
 {
 rlimit rl;
    getrlimit(RLIMIT_AS, &rl);
    total_memory = rl.rlim_cur;
 
    Avec::init();
+   LibPaths::init(argv0);
    Value::init();
 //   PrefixNode::print_all(cerr);
 }
@@ -119,134 +121,6 @@ signal_USR1_handler(int)
 static struct sigaction old_USR1_action;
 static struct sigaction new_USR1_action;
 
-//-----------------------------------------------------------------------------
-static char APL_bin_path[PATH_MAX + 1] = "";
-const char * get_APL_bin_path()   { return APL_bin_path; }
-
-static void
-set_APL_bin_path(const char * argv0)
-{
-   if (strchr(argv0, '/') == 0)
-      {
-         // if argv0 contains no / then realpath() seems to prepend the current
-         // directory to argv0 (which is wrong since argv0 may be in $PATH).
-         //
-         // we fix this by searching argv0 in $PATH
-         //
-         const char * path = getenv("PATH");   // must NOT be modified
-         
-         if (path)
-            {
-              const size_t alen = strlen(argv0);
-              const size_t plen = strlen(path);
-              char path1[plen + 1];
-              strncpy(path1, path, sizeof(path1));
-              char * next = path1;
-              for (;;)
-                  {
-                    char * semi = strchr(next, ':');
-                    if (semi)   *semi = 0;
-                    char filename[plen + alen + 10];
-                    snprintf(filename, sizeof(filename), "%s/%s",
-                             next, argv0);
-
-                    if (access(filename, X_OK) == 0)
-                       {
-                         strncpy(APL_bin_path, next, sizeof(APL_bin_path));
-                         return;
-                       }
-
-                    if (semi == 0)   break;
-                    next = semi + 1;
-                  }
-            }
-      }
-
-const void * unused = realpath(argv0, APL_bin_path);
-   APL_bin_path[PATH_MAX] = 0;
-char * slash =   strrchr(APL_bin_path, '/');
-   if (slash)   *slash = 0;
-
-   // if we have a PWD and it is a prefix of APL_bin_path then replace PWD
-   // by './'
-   //
-const char * PWD = getenv("PWD");
-   if (PWD)   // we have a pwd
-      {
-        const int PWD_len = strlen(PWD);
-        if (!strncmp(PWD, APL_bin_path, PWD_len) && PWD_len > 1)
-           {
-             strcpy(APL_bin_path + 1, APL_bin_path + PWD_len);
-             APL_bin_path[0] = '.';
-           }
-      }
-}
-//-----------------------------------------------------------------------------
-static char APL_lib_root[PATH_MAX + 10] = "";
-const char * get_APL_lib_root()   { return APL_lib_root; }
-
-/// return true if files dir/workspaces amd dir/wslib1 exist (not necessarily
-/// being direcories
-static bool
-is_lib_root(const char * dir)
-{
-char filename[PATH_MAX + 1];
-
-   snprintf(filename, sizeof(filename), "%s/workspaces", dir);
-   if (access(filename, F_OK))   return false;
-
-   snprintf(filename, sizeof(filename), "%s/wslib1", dir);
-   if (access(filename, F_OK))   return false;
-
-   return true;
-}
-
-static void
-set_APL_lib_root()
-{
-const char * path = getenv("APL_LIB_ROOT");
-   if (path)
-      {
-        const void * unused = realpath(path, APL_lib_root);
-        return;
-      }
-
-   // search from "." to "." for  a valid lib-root
-   //
-int last_len = 2*PATH_MAX;
-   APL_lib_root[0] = '.';
-   APL_lib_root[1] = 0;
-   for (;;)
-       {
-         const void * unused = realpath(APL_lib_root, APL_lib_root);
-         int len = strlen(APL_lib_root);
-
-         if (is_lib_root(APL_lib_root))   return;   // lib-rot found
-
-         // if length has not decreased then we stop in order to
-         // avoid an endless loop.
-         //
-         if (len >= last_len)   break;
-         last_len = len;
-
-         // move up by appending /..
-         //
-         APL_lib_root[len++] = '/';
-         APL_lib_root[len++] = '.';
-         APL_lib_root[len++] = '.';
-         APL_lib_root[len++] = 0;
-       }
-
-   // no lib-root found. use ".";
-   //
-const void * unused = realpath(".", APL_lib_root);
-}
-
-void
-set_APL_lib_root(const char * new_root)
-{
-const void * unused = realpath(new_root, APL_lib_root);
-}
 //-----------------------------------------------------------------------------
 /**
     When APL is started as a script. then several options might be grouped
@@ -397,7 +271,10 @@ const char * build_tag[] = { BUILDTAG, 0 };
             }
          out << *bt << endl;
        }
-   out << endl;
+
+   out << "  Readline:       " << HEX4(Input::readline_version()) << endl
+       << endl;
+
    Output::set_color_mode(Output::COLM_OUTPUT);
 }
 //-----------------------------------------------------------------------------
@@ -436,6 +313,13 @@ show_configure_options()
 #endif
    << endl <<
 
+#ifdef HAVE_LIBREADLINE
+   "    libreadline is used (default)"
+#else
+   "    libreadline is not used (disabled or not present)"
+#endif
+   << endl <<
+
    "    MAX_RANK_WANTED="     << MAX_RANK_WANTED
         << is_default(MAX_RANK_WANTED == 8)
    << endl <<
@@ -460,6 +344,7 @@ show_configure_options()
 #else
    "    VF_TRACING_WANTED=no (default)"
 #endif
+   << endl
    << endl;
 
 
@@ -548,12 +433,12 @@ show_GPL(ostream & out)
 //-----------------------------------------------------------------------------
 /// print a welcome message (copyright notice)
 static void
-show_welcome(ostream & out)
+show_welcome(ostream & out, const char * argv0)
 {
 char c1[200];
 char c2[200];
    snprintf(c1, sizeof(c1), _("Welcome to GNU APL version %s"),PACKAGE_VERSION);
-   snprintf(c2, sizeof(c2), _("for details run: %s --gpl."), progname);
+   snprintf(c2, sizeof(c2), _("for details run: %s --gpl."), argv0);
 
 const char * lines[] =
 {
@@ -746,14 +631,12 @@ int line = 0;
          char opt[BUFSIZE] = { 0 };
          char arg[BUFSIZE] = { 0 };
          int d[20];
-         const int count = sscanf(s, "%s" " %s %X %X %X %X"
-                                          " %X %X %X %X %X"
-                                          " %X %X %X %X %X"
-                                          " %X %X %X %X %X",
-                                     opt, arg,  d+ 1, d+ 2, d+ 3, d+ 4,
-                                          d+ 5, d+ 6, d+ 7, d+ 8, d+ 9,
-                                          d+10, d+11, d+12, d+13, d+14,
-                                          d+15, d+16, d+17, d+18, d+19);
+         const int count = sscanf(s, "%s"
+                   " %s %X %X %X %X %X %X %X %X %X"
+                   " %X %X %X %X %X %X %X %X %X %X",
+              opt, arg,  d+ 1, d+ 2, d+ 3, d+ 4, d+ 5, d+ 6, d+ 7, d+ 8, d+ 9,
+                   d+10, d+11, d+12, d+13, d+14, d+15, d+16, d+17, d+18, d+19);
+
          if (count < 2)
             {
               CERR << "bad tag or value at line " << line
@@ -812,6 +695,21 @@ int line = 0;
               loop(p, count - 1)   Output::clear_EOL[p] = (char)(d[p] & 0xFF);
               Output::clear_EOL[count - 1] = 0;
             }
+         else if (!strncasecmp(opt, "LIBREF-", 7))
+            {
+              const int lib_ref = atoi(opt + 7);
+              if (lib_ref < 0 || lib_ref > 9)
+                 {
+                   CERR << "bad library reference number " << lib_ref
+                        << " at line " << line << " of config file "
+                        << filename << " (ignored)" << endl;
+                   continue;
+                 }
+
+              LibPaths::set_lib_dir((LibRef)lib_ref, arg,
+                                    sys ? LibPaths::LibDir::CS_PREF_SYS
+                                        : LibPaths::LibDir::CS_PREF_HOME);
+            }
        }
 
    fclose(f);
@@ -824,14 +722,13 @@ const char ** argv = expand_argv(argc, _argv);
    Quad_ARG::argc = argc;   // remember argc for ⎕ARG
    Quad_ARG::argv = argv;   // remember argv for ⎕ARG
 
+const char * argv0 = argv[0];
+   init(argv0);
+
 user_preferences up;
    read_config_file(true,  up);   // /etc/gnu-apl.d/preferences
    read_config_file(false, up);   // $HOME/.gnu_apl
 
-   progname = argv[0];
-
-   set_APL_bin_path(progname);
-   set_APL_lib_root();
 
    // struct sigaction differs between GNU/Linux and other systems, which
    // causes direct bracket assignment to not compile on some machines.
@@ -851,8 +748,6 @@ user_preferences up;
    sigaction(SIGUSR1, &new_USR1_action,      &old_USR1_action);
 // sigaction(SIGSEGV, &new_segfault_action,  &old_segfault_action);
 
-   init();
-
 Workspace w;
 
    // init NLS so that usage() will be translated
@@ -1051,7 +946,7 @@ Workspace w;
 
    if (up.wait_ms)   usleep(1000*up.wait_ms);
 
-   if (!silent)   show_welcome(cout);
+   if (!silent)   show_welcome(cout, argv0);
 
    Log(LOG_startup)   CERR << "PID is " << getpid() << endl;
    Log(LOG_argc_argv)   show_argv(argc, argv);
@@ -1069,10 +964,10 @@ Workspace w;
    if (up.do_CONT)
       {
          UCS_string cont("CONTINUE.xml");
-         vector<UCS_string> lib_file;
-         lib_file.push_back(cont);
-         UTF8_string path = Command::get_lib_file_path(lib_file);
-         if (access((const char *)path.c_str(), F_OK) == 0)
+         UTF8_string filename =
+            LibPaths::get_lib_filename(LIB0, "CONTINUE", true, "xml");
+
+         if (access((const char *)filename.c_str(), F_OK) == 0)
             {
               UCS_string load_cmd(")LOAD CONTINUE");
               Command::process_line(load_cmd);
